body
{
	margin: 0;
	padding: 0;
}

.blue_center
{
	background-color: #6293cd;
	font-weight: bold;
	text-align: center;
}

.error
{
	color: red;
	font-weight: bold;
}

.hidden_div
{
	background-color: #6293cd;
	border-style: solid;
	border-width: 1px;
	display: none;
	text-align: center;
}
